/*
? [2024] Microchip Technology Inc. and its subsidiaries.

    Subject to your compliance with these terms, you may use Microchip 
    software and any derivatives exclusively with Microchip products. 
    You are responsible for complying with 3rd party license terms  
    applicable to your use of 3rd party software (including open source  
    software) that may accompany Microchip software. SOFTWARE IS ?AS IS.? 
    NO WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS 
    SOFTWARE, INCLUDING ANY IMPLIED WARRANTIES OF NON-INFRINGEMENT,  
    MERCHANTABILITY, OR FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT 
    WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE, 
    INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY 
    KIND WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF 
    MICROCHIP HAS BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE 
    FORESEEABLE. TO THE FULLEST EXTENT ALLOWED BY LAW, MICROCHIP?S 
    TOTAL LIABILITY ON ALL CLAIMS RELATED TO THE SOFTWARE WILL NOT 
    EXCEED AMOUNT OF FEES, IF ANY, YOU PAID DIRECTLY TO MICROCHIP FOR 
    THIS SOFTWARE.
*/
#include "mcc_generated_files/system/system.h"
#include "mcc_generated_files/main_core/main_core.h"
#include "mcc_generated_files/system/pins.h"
#include "mcc_generated_files/timer/tmr1.h"
#include "mcc_generated_files/pwm/sccp1.h"
#include "mcc_generated_files/cmp/cmp1.h"

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#define DATA_UNDER_TEST 0xAAAA        // data used for protocol write and read test

const struct MAIN_CORE_INTERFACE *mainCore = &MSIInterface;
float Pwm_sine_value = 0;
float DAC_data = 0;
int DAC_data_int = 0;
int Middle_of_DAC = 0;
int PWM_Dutycycle = 0;

uint16_t dataReceive[MSI1_ProtocolA_SIZE];    //for Protocol size refer main_core_types.h file
uint16_t dataSend[MSI1_ProtocolB_SIZE];

/*
    Main application
*/

void Sec_Interrupt(void) 
{
    LED3_Toggle();
}

float sine_table[180] =
{
    0.0000, 0.0175, 0.0349, 0.0523, 0.0698, 0.0872, 0.1045, 0.1219, 0.1392, 0.1564,
    0.1736, 0.1908, 0.2079, 0.2250, 0.2419, 0.2588, 0.2756, 0.2924, 0.3090, 0.3256, 
    0.3420, 0.3584, 0.3746, 0.3907, 0.4067, 0.4226, 0.4384, 0.4540, 0.4695, 0.4848, 
    0.5000, 0.5150, 0.5299, 0.5446, 0.5592, 0.5736, 0.5878, 0.6018, 0.6157, 0.6293, 
    0.6428, 0.6561, 0.6691, 0.6820, 0.6947, 0.7071, 0.7193, 0.7314, 0.7431, 0.7547,
    0.7660, 0.7771, 0.7880, 0.7986, 0.8090, 0.8192, 0.8290, 0.8387, 0.8480, 0.8572,
    0.8660, 0.8746, 0.8829, 0.8910, 0.8988, 0.9063, 0.9135, 0.9205, 0.9272, 0.9336,
    0.9397, 0.9455, 0.9511, 0.9563, 0.9613, 0.9659, 0.9703, 0.9744, 0.9781, 0.9816,
    0.9848, 0.9877, 0.9903, 0.9925, 0.9945, 0.9962, 0.9976, 0.9986, 0.9994, 0.9998,
    0.9998, 0.9994, 0.9986, 0.9976, 0.9962, 0.9945, 0.9925, 0.9903, 0.9877, 0.9848,
    0.9816, 0.9781, 0.9744, 0.9703, 0.9659, 0.9613, 0.9563, 0.9511, 0.9455, 0.9397,
    0.9336, 0.9272, 0.9205, 0.9135, 0.9063, 0.8988, 0.8910, 0.8829, 0.8746, 0.8660,
    0.8572, 0.8480, 0.8387, 0.8290, 0.8192, 0.8090, 0.7986, 0.7880, 0.7771, 0.7660,
    0.7547, 0.7431, 0.7314, 0.7193, 0.7071, 0.6947, 0.6820, 0.6691, 0.6561, 0.6428,
    0.6293, 0.6157, 0.6018, 0.5878, 0.5736, 0.5592, 0.5446, 0.5299, 0.5150, 0.5000,
    0.4848, 0.4695, 0.4540, 0.4384, 0.4226, 0.4067, 0.3907, 0.3746, 0.3584, 0.3420,
    0.3256, 0.3090, 0.2924, 0.2756, 0.2588, 0.2419, 0.2250, 0.2079, 0.1908, 0.1736,
    0.1564, 0.1392, 0.1219, 0.1045, 0.0872, 0.0698, 0.0523, 0.0349, 0.0175, 0.0000,
};

int main(void)
{
    SYSTEM_Initialize();
    
    //Wait for interrupt from Main core 
    while(!MAIN_CORE_IsInterruptRequested());
    MAIN_CORE_InterruptRequestAcknowledge();
    while(MAIN_CORE_IsInterruptRequested());
    MAIN_CORE_InterruptRequestAcknowledgeComplete();
    
    //Mailbox read    
    MAIN_CORE_ProtocolRead(MSI1_ProtocolA, (uint16_t*)dataReceive);
    //Copy the received data for retransmission
    memcpy(dataSend, dataReceive, sizeof(dataReceive));
    //Retransmits Mailbox 
    MAIN_CORE_ProtocolWrite(MSI1_ProtocolB, (uint16_t*)dataSend);
    
    //Issue interrupt to Main core
    MAIN_CORE_InterruptRequestGenerate();
    while(!MAIN_CORE_IsInterruptRequestAcknowledged());
    MAIN_CORE_InterruptRequestComplete();
    while(MAIN_CORE_IsInterruptRequestAcknowledged());

    //Glow LED on data match
    if(dataReceive[0] == DATA_UNDER_TEST)
    {
        LED3_SetHigh();
    }
    else
    {
        LED3_SetLow();
    }
    
    TMR1_TimeoutCallbackRegister(Sec_Interrupt);
    
//    Middle_of_DAC = (3890-205)/2;

    while(1)
    {

        //sin波
        for (int m=0; m<180; ++m)
//        {
//            Pwm_sine_value = sine_table[m] * 100;
//            SCCP1_PWM_DutyCycleSet(Pwm_sine_value);
//            DAC_data = (((Middle_of_DAC + 1) / 100) * Pwm_sine_value) + 2048;
//            DAC_data_int = (int)DAC_data;
//            CMP1_DACDataWrite(DAC_data_int);
//       }
        
//        for (int n=0; n<180; ++n)
//        {
//            Pwm_sine_value = sine_table[n] * 100;
//            SCCP1_PWM_DutyCycleSet(Pwm_sine_value);
//            DAC_data = ((Middle_of_DAC / 100) * (100-Pwm_sine_value)) + 205;
//            DAC_data_int = (int)DAC_data;
//            CMP1_DACDataWrite(DAC_data_int);
//        }
        
        MAIN_CORE_ProtocolRead(MSI1_ProtocolA, (uint16_t*)dataReceive);
        SCCP1_PWM_DutyCycleSet((400 * ((dataReceive[0]*10)+dataReceive[1])) / 100);

    }    
}